#define VERSION "0.8"

/* Icy headers */
typedef struct icyHeaders {
  char *name;
  char *genre;
  char *notice1;
  char *notice2;
  char *content_type;
  char *url;
  /* type: 
   * 0 Shoutcast 
   * 1 Icecast 
   * 2 Icecast2 
   */
  int type; 
  int br;
  int metaint;
  int pub;
} icyHeaders;

/* Currently played song */
typedef struct currentSong {
  char *title;
  char *artist;
} currentSong;

/* Command line arguments are stored here */
typedef struct commandLine {
  char *dir;
  char *host;
  char *mountpoint;
  char *logFile;
  unsigned int port;
  unsigned short pige;
  unsigned short quite;
  unsigned short live;
  unsigned short background;
} commandLine;
  
typedef struct serverSettings {
  unsigned int port;
  char *serverAddr;
  char *mountpoint;
} serverSettings;

typedef struct stats {
  unsigned long songs;
  unsigned long reconnections;
} stats;

typedef struct cPigeUptime {
  unsigned int day;
  unsigned int hour;
  unsigned int min;
  unsigned int sec;
} cPigeUptime;

/* explicit ? */
void print_credits ();
int server_socket = -1;

/* TCP connection to the server */
int server_connect (char *servername, int serverport);

/* Disconnect */
int server_close (int serversocket);

/* fetch metadata from mp3 stream */
char *getHeaders(int serversocket);

/* fetch song title from metadata */
char *getTitle(char *titleString);

/* parse metadata */
char *readMeta(int serversocket);

/* return a string pointer to print informations about current stream,
 * like title, bytes received, ... 
 */
char *statusLine(long long unsigned int uptime, long long unsigned int metacounter, int metaint, char *titre, char *nexttitre);

/* return a pointer to 128bytes of id3 tags */
char *GetId3v1(char *titre, char *artiste, char *radioName);

/* fetch icy data from stream */
icyHeaders *readicyheaders(char *headers);

/* store informations about title currently played (for id3v1) */
currentSong *getCurrentSong(char *titre);

/* print icy informations */
void print_icyheaders(icyHeaders *_icy_headers );

/* send http headers to connect to remote stream */
int sendHeaders(int serversocket, char *mountpoint, int metadata);

/* connect / reconnect to a stream */
int reconnect(char *servername, int serverport, int time, int _nb_tentatives, int _get_headers);

/* Time helpers */
int getHour();
int getDay();
int getMinute();
int getMonth();
int getYear();

/* Parse command line arguments */
commandLine *parseCommandLine(int argc, char **argv);
void testCommandLine();

/* print help ... */
void print_help();

/* Usefull for debug */
void *memory_allocation(size_t size);

/* get Return code from HTTP server */
int getHTTPCode(char *buffer);

serverSettings *parseURL(char *url);
void printServSettings(serverSettings *set);

/* getUptime */
cPigeUptime *getUptime(long long unsigned int uptime);
char *getStats(long long unsigned int uptime, long long unsigned int metacounter, int metaint);

stats *cPigeStats;
